# -*- coding: utf-8 -*-

import os
import sys
import tkinter as tk
from tkinter import messagebox as msgb
import asyncio
import sqlite3


class App:
    async def exec(self):
        self.window = Win(asyncio.get_event_loop())
        await self.window.show()
        
    
class Win(tk.Tk):
    def __init__(self, loop: callable) -> None:
        super().__init__()
        self.protocol('WM_DELETE_WINDOW', self.__on_close)
        self.loop = loop
        self.conn = None
        self.win_still = True
        self.still = False
        self.nick = 'trescon'
        self._populate()

    def _populate(self) -> None:
        self.title('Esempio ASINCRONO per @trescon')
        lbl = tk.Label(self, text='Messaggi rilevati:')
        lbl.grid(row=0, column=0, columnspan=2, padx=5, pady=5, sticky='w')
        self.t_msg = tk.Text(self)
        self.t_msg.grid(row=1, column=0, padx=(5,2), pady=5, sticky='nsew')
        vscroll = tk.Scrollbar(self, orient=tk.VERTICAL,
                               command=self.t_msg.yview)
        vscroll.grid(row=1, column=1, padx=(2,5), pady=5, sticky='ns')
        self.t_msg.configure(yscrollcommand=vscroll.set)
        pn_cmd = tk.Frame(self)
        pn_cmd.grid(row=2, column=0, columnspan=2, padx=5, pady=5, sticky='ew')
        self.bt_connect = tk.Button(pn_cmd, text='Connetti al database',
                                    command=self._on_connect)
        self.bt_connect.grid(row=0, column=0, padx=2, pady=5, sticky='ew')
        bt_close = tk.Button(pn_cmd, text='Chiudi Applicazione',
                             command=self.__on_close)
        bt_close.grid(row=0, column=1, padx=2, pady=5, sticky='ew')
        pn_cmd.grid_columnconfigure(0, weight=1, uniform='cmd')
        pn_cmd.grid_columnconfigure(1, weight=1, uniform='cmd')
        self.grid_columnconfigure(0, weight=1)
        self.grid_rowconfigure(1, weight=1)

    async def show(self):
        while self.win_still:
            self.update()
            await asyncio.sleep(.1)
            
    def __on_close(self) -> None:
        if self.conn:
            self.win_still = False
            self.still = False
        else:
            self.win_still = False
        self.after_idle(self.destroy)

    def _on_connect(self) -> None:
        if self.conn: return
        appdir = os.path.abspath(os.path.dirname(sys.argv[0]))
        db_name = os.path.join(appdir, 'for_trescon.db')
        try:
            self.conn = sqlite3.connect(db_name)
            self.still = True
        except sqlite3.IntegrityError as e:
            txt = traceback.print_exc()
            msgb.showerror('Errore dati', txt)
            return
        self.bt_connect.configure(state='disabled')
        self.loop.create_task(self.read_messages())
        
    async def read_messages(self) -> None:
        while self.still:
            query = '''SELECT * FROM messages A
WHERE NOT EXISTS(SELECT * FROM messages_read B
                 WHERE B.message_id = A.id
                 AND B.readernick = ?)
ORDER BY a.ID'''
            res = self.conn.execute(query, (self.nick,))
            if res:
                for r in res.fetchall():
                    text = ' '.join([str(x) for x in r])
                    try:
                        query = 'INSERT INTO messages_read VALUES(?, ?)'
                        result = self.conn.execute(query, (r[0], self.nick))
                        self.conn.commit()
                    except sqlite3.IntegrityError as e:
                        txt = traceback.print_exc()
                        msgb.showerror('Errore dati', txt)
                        break
                    self.show_message(text)
            await asyncio.sleep(10)

    def show_message(self, msg: str) -> None:
        txt = msg + '\n'
        self.t_msg.insert(tk.END, txt)


if __name__ == '__main__':
    asyncio.run(App().exec())
